<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */
class Shopgate_Models_Catalog_Category extends Shopgate_Model_Catalog_Category
{
    protected $config;
    protected $log;

    /**
     * @param ShopgateConfigXtc $config
     */
    public function __construct($config)
    {
        $this->config = $config;
        $this->log    = ShopgateLogger::getInstance();
        parent::__construct();
    }

    /**
     * Generate a list of category uids which contain the committed product
     *
     * @param $item
     * @param $maxSort
     * @param $addToOrderIndex
     *
     * @return array
     * @throws ShopgateLibraryException
     */
    public function getProductCategoryNumbers($item, $maxSort, $addToOrderIndex)
    {
        $this->log("execute getProductCategoryNumbers() ...", ShopgateLogger::LOGTYPE_DEBUG);

        $categoryReducementMap = $this->getCategoryReducementMap();
        $categoryNumbers       = array();
        $catsQry               = "SELECT DISTINCT ptc.categories_id, c.products_sorting2 
                                  FROM " . TABLE_PRODUCTS_TO_CATEGORIES . " ptc
                                  INNER JOIN " . TABLE_CATEGORIES . " c ON (ptc.categories_id = c.categories_id)
                                  WHERE ptc.products_id = '" . $item["products_id"] . "' AND c.categories_status = 1";
        $catsQuery             = xtc_db_query($catsQry);

        while ($category = xtc_db_fetch_array($catsQuery)) {
            if (empty($category["categories_id"])) {
                continue;
            }

            if ($category["products_sorting2"] != "ASC") {
                if ($this->config->getReverseItemsSortOrder()) {
                    $sort = $maxSort - $item["products_sort"];
                } else {
                    $sort = $item["products_sort"];
                }

            } else {
                if ($this->config->getReverseItemsSortOrder()) {
                    $sort = $item["products_sort"];
                } else {
                    $sort = $maxSort - $item["products_sort"];
                }
            }

            if (!empty($sort) || ((string)$sort === '0')) {
                $sort += $addToOrderIndex;
            }

            $categoryNumbers[] = array(
                'id'   => $category["categories_id"],
                'sort' => $sort
            );
        }

        // check if there is a category replacement map to reduce categories depth
        if (!empty($categoryReducementMap)) {
            foreach ($categoryNumbers as &$categoryNumber) {
                if (isset($categoryReducementMap[$categoryNumber['id']])) {
                    $categoryNumber['id'] = $categoryReducementMap[$categoryNumber['id']];
                }
            }
        }

        return $categoryNumbers;
    }

    /**
     * Returns all sub categories including the given parent as a list that is a mapping
     * from one category to a higher category if a given depth is exceeded
     *
     * @param int $maxDepth
     * @param int $parentId
     * @param int $copyId
     * @param int $depth
     *
     * @throws ShopgateLibraryException
     * @return array
     */
    public function getCategoryReducementMap($maxDepth = null, $parentId = null, $copyId = null, $depth = null)
    {

        $this->log("execute _getCategoryReducementMap() ...", ShopgateLogger::LOGTYPE_DEBUG);

        if (empty($maxDepth)) {
            return array();
        }

        $circularDepthStop = 50;

        if (empty($depth)) {
            $depth = 1;
        } elseif ($depth > $circularDepthStop) {
            // disallow circular category connections (detect by a maximum depth)
            throw new ShopgateLibraryException(
                ShopgateLibraryException::PLUGIN_DATABASE_ERROR,
                'error on loading sub-categories: Categories-Depth exceedes a value of ' . $circularDepthStop
                . '. Check if there is a circular connection (referenced categories ids: ' . $parentId . '=>)',
                true
            );
        }

        // select by parent id, if set
        $qry = "SELECT `categories_id` FROM `" . TABLE_CATEGORIES . "` WHERE" .
            (!empty($parentId)
                ? " (`parent_id` = '{$parentId}')"
                : " (`parent_id` IS NULL OR `parent_id` = 0 OR `parent_id` = '')");

        $qryResult = xtc_db_query($qry);

        if (!$qryResult) {
            throw new ShopgateLibraryException(
                ShopgateLibraryException::PLUGIN_DATABASE_ERROR,
                'error on selecting categories',
                true
            );
        }

        // add all sub categories to a simple one-dimensional array
        $categoryMap = array();

        while ($row = xtc_db_fetch_array($qryResult)) {
            // copy only if a maximum depth is set, yet
            if (!empty($maxDepth)) {
                if ($depth == $maxDepth) {
                    $copyId = $row['categories_id'];
                }
            }
            // Check if a mapping to a higher category needs to be applied
            if (!empty($copyId) && !empty($row['categories_id'])) {
                $categoryMap[$row['categories_id']] = $copyId;
            } else {
                // no mapping to other categories, map to itself!
                $categoryMap[$row['categories_id']] = $row['categories_id'];
            }

            $subCategories = $this->getCategoryReducementMap($maxDepth, $row['categories_id'], $copyId, $depth + 1);

            if (!empty($subCategories)) {
                $categoryMap = $categoryMap + $subCategories;
            }
        }

        return $categoryMap;
    }

    /**
     * Return true if category is blacklisted
     *
     * @param int $id
     *
     * @return bool
     * @throws ShopgateLibraryException
     */
    public function isCategoryBlacklisted($id)
    {
        $categoryBlacklist = $this->getBlacklistedCategoryIds();
        $categoryList      = $this->buildCategoryChildren($id);

        foreach ($categoryBlacklist as $bcId) {
            foreach ($categoryList as $cId) {
                if ($bcId == $cId) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Return an array with all category ids which are blacklisted
     *
     * @return array
     * @throws ShopgateLibraryException
     */
    public function getBlacklistedCategoryIds()
    {
        $categoryBlacklist = $this->config->getCategoryBlacklistIds();
        if (is_null($categoryBlacklist) || count($categoryBlacklist) == 0) {
            return array();
        }

        $bQuery = "SELECT c.categories_id FROM " . TABLE_CATEGORIES . " AS c WHERE ";
        $i      = 1;

        foreach ($categoryBlacklist as $item) {
            $bQuery .= " c.categories_id = \"" . $item . "\" " . ($i < count($categoryBlacklist) ? " OR " : " ");
            $i++;
        }

        $bQuery .= " GROUP BY c.categories_id;";
        $result = xtc_db_query($bQuery);

        $i      = 0;
        $retval = array();

        while ($category = xtc_db_fetch_array($result)) {
            $tmp = $this->buildCategoryChildren($category['categories_id']);
            foreach ($tmp as $value) {
                $retval[$i] = $value;
                $i++;
            }
        }

        return $retval;
    }

    /**
     * Return list of child category ids including the given parent.
     * If $maxdepth value is reached, an exception will be thrown.
     *
     * @param int   $parentId
     * @param array $catList
     * @param int   $depth
     * @param int   $maxdepth
     *
     * @return array
     * @throws ShopgateLibraryException
     */
    private function buildCategoryChildren($parentId = 0, $catList = array(), $depth = 1, $maxdepth = 500)
    {

        $this->log(
            'Start buldiding Categories children list: parent_id = ' . $parentId . '...',
            ShopgateLogger::LOGTYPE_DEBUG
        );

        if ($depth < 1) {
            $depth = 1;
        }

        if ($parentId < 0 || empty($parentId) || is_null($parentId)) {
            return $catList;
        }

        if ($depth >= $maxdepth) {
            // disallow circular category connections (detect by a maximum depth)
            throw new ShopgateLibraryException(
                ShopgateLibraryException::PLUGIN_DATABASE_ERROR,
                'error on loading sub-categories: Categories-Depth exceedes a value of ' . $maxdepth
                . '. Check if there is a circular connection (referenced categories ids: ' . $parentId . '=>)',
                true
            );
        }

        if (count($catList) == 0) {
            $catList[0] = $parentId;
        }

        $qry = "SELECT DISTINCT	c.categories_id, c.parent_id FROM " . TABLE_CATEGORIES . " AS c
				WHERE c.parent_id = " . $parentId . " ORDER BY c.categories_id ASC";

        $qry = xtc_db_query($qry);

        while ($item = xtc_db_fetch_array($qry)) {
            $tmpList[] = $item['categories_id'];
            $catList[] = $item['categories_id'];
        }

        if (!is_null($tmpList)) {
            foreach ($tmpList as $catid) {
                return $this->buildCategoryChildren($catid, $catList, $depth + 1, $maxdepth);
            }
        }

        return $catList;
    }
}
