<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Xtcommerce3\Tracking\Service;

use Shopgate\FacebookPixelLibrary\Event\Event\AddToCart;
use Shopgate\FacebookPixelLibrary\Event\Event\InitiateCheckout;
use Shopgate\FacebookPixelLibrary\Event\Event\Purchase;
use Shopgate\FacebookPixelLibrary\Event\Event\ViewContent;
use Shopgate\FacebookPixelLibrary\Event\Service\EventRenderer as FacebookEventRenderer;
use Shopgate\FacebookPixelLibrary\Event\ValueObject\ContentId;
use Shopgate\FacebookPixelLibrary\Event\ValueObject\ContentName;
use Shopgate\FacebookPixelLibrary\Event\ValueObject\Currency;
use Shopgate\FacebookPixelLibrary\Event\ValueObject\NumItems;
use Shopgate\FacebookPixelLibrary\Event\ValueObject\Value;
use Shopgate\Xtcommerce3\Tracking\Entity\Cart;
use Shopgate\Xtcommerce3\Tracking\Repository\EventData;

class EventRenderer
{
    /** @var FacebookEventRenderer */
    private $renderer;

    /** @var EventData */
    private $eventDataRepository;

    /**
     * @param FacebookEventRenderer $renderer
     * @param EventData             $eventDataRepository
     */
    public function __construct(
        FacebookEventRenderer $renderer,
        EventData $eventDataRepository
    ) {
        $this->renderer            = $renderer;
        $this->eventDataRepository = $eventDataRepository;
    }

    /**
     * @return string
     */
    public function getSnippet()
    {
        $viewContentEvent = new ViewContent(
            null,
            new Currency($this->eventDataRepository->getCurrencyCode()),
            new ContentName($this->eventDataRepository->getPageName()),
            null,
            array()
        );

        $events     = array();
        $shadowCart = $this->eventDataRepository->getShadowCart();
        $this->addAddToCartEvent($events, $shadowCart);

        switch ($this->eventDataRepository->getScriptName()) {
            case "product_info.php":
                $viewContentEvent = new ViewContent(
                    new Value($this->formatPrice($this->eventDataRepository->getCurrentProductPrice() / 100)),
                    new Currency($this->eventDataRepository->getCurrencyCode()),
                    new ContentName($this->eventDataRepository->getCurrentProductName()),
                    $this->eventDataRepository->getCurrentProductContentType(),
                    array(new ContentId($this->eventDataRepository->getCurrentProductId()))
                );
                break;

            case "checkout_shipping.php":
                $events[] = new InitiateCheckout(
                    new Value($this->formatPrice($this->eventDataRepository->getNativeCart()->show_total())),
                    new Currency($this->eventDataRepository->getCurrencyCode()),
                    null,
                    null,
                    $this->getProductIdsFromNativeCart($this->eventDataRepository->getNativeCart()),
                    new NumItems($this->eventDataRepository->getNativeCart()->count_contents())
                );
                break;
            case "checkout_success.php":
                $events[] = new Purchase(
                    new Value($this->formatPrice($shadowCart->getAmountItems() / 100)),
                    new Currency($this->eventDataRepository->getCurrencyCode()),
                    null,
                    null,
                    $this->getProductIdsFromShadowCart($shadowCart),
                    new NumItems($this->getNumberOfItemsFromShadowCart($shadowCart))
                );
                break;
            default:
        }

        // update and persist the shadow cart with its latest changes
        $shadowCart->update($this->eventDataRepository->getNativeCart());
        $this->eventDataRepository->persistShadowCart($shadowCart);

        // prepend the ViewContent event to the list of events that should be rendered
        array_unshift($events, $viewContentEvent);

        return $this->renderer->render($events);
    }

    /**
     * @param \shoppingCart $xtc3Cart
     *
     * @return ContentId[]
     */
    private function getProductIdsFromNativeCart(\shoppingCart $xtc3Cart)
    {
        $ids = array();
        foreach ($xtc3Cart->get_products() as $xtc3Product) {
            $ids[] = new ContentId((string)$xtc3Product['id']);
        }

        return $ids;
    }

    /**
     * @param Cart $shadowCart
     *
     * @return ContentId[]
     */
    private function getProductIdsFromShadowCart(Cart $shadowCart)
    {
        $ids = array();

        foreach ($shadowCart->getItems() as $item) {
            $ids[] = new ContentId((string)$item->id);
        }

        return $ids;
    }

    /**
     * @param Cart $shadowCart
     *
     * @return int
     */
    private function getNumberOfItemsFromShadowCart(Cart $shadowCart)
    {
        $numberOfItems = 0;
        foreach ($shadowCart->getItems() as $item) {
            $numberOfItems += $item->quantity;
        }

        return $numberOfItems;
    }

    /**
     * @param array $events
     * @param Cart  $shadowCart
     *
     * @post $events has an object of type AddToCart appended in case there are differences between the current and the
     *       shadow cart.
     */
    private function addAddToCartEvent(array &$events, Cart $shadowCart)
    {
        $differences = $shadowCart->getDifferences($this->eventDataRepository->getNativeCart());

        if (empty($differences)) {
            return;
        }

        $ids    = array();
        $amount = 0;
        foreach ($differences as $difference) {
            if ($difference->quantityDifference <= 0) {
                continue;
            }

            $ids[]  = new ContentId((string)$difference->itemId);
            $amount += $difference->amountDifference;
        }

        if (empty($ids)) {
            return;
        }

        $events[] = new AddToCart(
            new Value($this->formatPrice($amount / 100)),
            new Currency($this->eventDataRepository->getCurrencyCode()),
            null,
            $this->eventDataRepository->getCurrentProductContentType(),
            $ids
        );
    }

    /**
     * @param int $price
     *
     * @return string
     */
    private function formatPrice($price)
    {
        return number_format($price, 2, '.', '');
    }
}