<?php

namespace Shopgate\FacebookPixelLibrary\Event\Service;

use Shopgate\FacebookPixelLibrary\Base\Repository\Settings;
use Shopgate\FacebookPixelLibrary\Event\Event\FacebookTrackingEvent;
use Shopgate\FacebookPixelLibrary\Event\Event\AddPaymentInfo;
use Shopgate\FacebookPixelLibrary\Event\Event\AddToCart;
use Shopgate\FacebookPixelLibrary\Event\Event\AddToWishlist;
use Shopgate\FacebookPixelLibrary\Event\Event\CompleteRegistration;
use Shopgate\FacebookPixelLibrary\Event\Event\InitiateCheckout;
use Shopgate\FacebookPixelLibrary\Event\Event\Lead;
use Shopgate\FacebookPixelLibrary\Event\Event\Purchase;
use Shopgate\FacebookPixelLibrary\Event\Event\Search;
use Shopgate\FacebookPixelLibrary\Event\Event\ViewContent;
use Shopgate\FacebookPixelLibrary\Event\ValueObject\TrackingType;
use Shopgate\FacebookPixelLibrary\Event\ValueObject\ContentId;

class JavaScriptTag implements EventRenderer
{
    /** @var EventRenderer */
    private $noscriptContentRenderer;

    /** @var Settings */
    private $settingsRepository;

    /** @var TrackingType */
    private $trackingType;

    /**
     * @param EventRenderer     $noscriptContentRenderer
     * @param Settings          $settingsRepository
     * @param TrackingType|null $trackingType
     */
    public function __construct(
        EventRenderer $noscriptContentRenderer,
        Settings $settingsRepository,
        TrackingType $trackingType = null
    ) {
        $this->noscriptContentRenderer = $noscriptContentRenderer;
        $this->settingsRepository      = $settingsRepository;
        $this->trackingType            = empty($trackingType) ? new TrackingType(TrackingType::TRACK) : $trackingType;
    }

    public function render(array $events)
    {
        $facebookPixelId = $this->settingsRepository->getFacebookPixelId();
        if (empty($facebookPixelId)) {
            return '';
        }

        $eventSnippets = '';
        /** @var FacebookTrackingEvent $event */
        foreach ($events as $event) {
            $eventSnippets .= $this->renderEvent($event);
        }
        if (empty($eventSnippets)) {
            return '';
        }

        return $this->getOpenCommentTag()
            . $this->getOpenScriptTag()
            . $this->getFacebookIncludeFunction()
            . $this->getFacebookInitSnippet($facebookPixelId)
            . $eventSnippets
            . $this->getCloseScriptTag()
            . $this->getNoScriptTagWithContent($events)
            . $this->getCloseCommentTag();
    }


    /**
     * @param FacebookTrackingEvent $event
     *
     * @return string
     */
    protected function renderEvent(FacebookTrackingEvent $event)
    {
        $eventTag          = '';
        $fullClassName     = get_class($event);
        $explodedClassName = explode('\\', $fullClassName);
        $className         = array_pop($explodedClassName);
        $renderFunction    = 'render' . $className;
        $isActiveFunction  = 'getFacebookPixel' . $className . 'EventActive';
        if ($this->settingsRepository->$isActiveFunction()) {
            $eventTag = "fbq('" . $this->trackingType->getString() . "', '" . $className . "', {"
                . $this->$renderFunction($event) . "});\n";
        }

        return $eventTag;
    }

    /**
     * @param AddPaymentInfo $event
     *
     * @return string
     */
    protected function renderAddPaymentInfo(AddPaymentInfo $event)
    {
        $contentFields = array();

        if (!empty($event->value)) {
            $contentFields[] = "value: " . $event->value->getString();
        }
        if (!empty($event->currency)) {
            $contentFields[] = "currency: '" . $event->currency->getString() . "'";
        }
        if (!empty($event->content_category)) {
            $contentFields[] = "content_category: '" . $event->content_category->getString() . "'";
        }
        if (!empty($event->content_ids)) {
            $contentFields[] =
                "content_ids: [" . $this->getContentIdsCommaSeparated($event->content_ids) . "]";
        }

        return implode(', ', $contentFields);
    }

    /**
     * @param AddToCart $event
     *
     * @return string
     */
    protected function renderAddToCart(AddToCart $event)
    {
        $contentFields = array();

        if (!empty($event->value)) {
            $contentFields[] = "value: " . $event->value->getString();
        }
        if (!empty($event->currency)) {
            $contentFields[] = "currency: '" . $event->currency->getString() . "'";
        }
        if (!empty($event->content_name)) {
            $contentFields[] = "content_name: '" . $event->content_name->getString() . "'";
        }
        if (!empty($event->content_type)) {
            $contentFields[] = "content_type: '" . $event->content_type->getString() . "'";
        }
        if (!empty($event->content_ids)) {
            $contentFields[] =
                "content_ids: [" . $this->getContentIdsCommaSeparated($event->content_ids) . "]";
        }

        return implode(', ', $contentFields);
    }

    /**
     * @param AddToWishlist $event
     *
     * @return string
     */
    protected function renderAddToWishlist(AddToWishlist $event)
    {
        $contentFields = array();

        if (!empty($event->value)) {
            $contentFields[] = "value: " . $event->value->getString();
        }
        if (!empty($event->currency)) {
            $contentFields[] = "currency: '" . $event->currency->getString() . "'";
        }
        if (!empty($event->content_name)) {
            $contentFields[] = "content_name: '" . $event->content_name->getString() . "'";
        }
        if (!empty($event->content_category)) {
            $contentFields[] = "content_category: '" . $event->content_category->getString() . "'";
        }
        if (!empty($event->content_ids)) {
            $contentFields[] =
                "content_ids: [" . $this->getContentIdsCommaSeparated($event->content_ids) . "]";
        }

        return implode(', ', $contentFields);
    }

    /**
     * @param CompleteRegistration $event
     *
     * @return string
     */
    protected function renderCompleteRegistration(CompleteRegistration $event)
    {
        $contentFields = array();

        if (!empty($event->value)) {
            $contentFields[] = "value: " . $event->value->getString();
        }
        if (!empty($event->currency)) {
            $contentFields[] = "currency: '" . $event->currency->getString() . "'";
        }
        if (!empty($event->content_name)) {
            $contentFields[] = "content_name: '" . $event->content_name->getString() . "'";
        }
        if (!empty($event->status)) {
            $contentFields[] = "status: '" . $event->status->getString() . "'";
        }

        return implode(', ', $contentFields);
    }

    /**
     * @param InitiateCheckout $event
     *
     * @return string
     */
    protected function renderInitiateCheckout(InitiateCheckout $event)
    {
        $contentFields = array();

        if (!empty($event->value)) {
            $contentFields[] = "value: " . $event->value->getString();
        }
        if (!empty($event->currency)) {
            $contentFields[] = "currency: '" . $event->currency->getString() . "'";
        }
        if (!empty($event->content_name)) {
            $contentFields[] = "content_name: '" . $event->content_name->getString() . "'";
        }
        if (!empty($event->content_category)) {
            $contentFields[] = "content_category: '" . $event->content_category->getString() . "'";
        }
        if (!empty($event->content_ids)) {
            $contentFields[] =
                "content_ids: [" . $this->getContentIdsCommaSeparated($event->content_ids) . "]";
        }
        if (!empty($event->num_items)) {
            $contentFields[] = "num_items: " . $event->num_items->getInt();
        }

        return implode(', ', $contentFields);
    }

    /**
     * @param Lead $event
     *
     * @return string
     */
    protected function renderLead(Lead $event)
    {
        $contentFields = array();

        if (!empty($event->value)) {
            $contentFields[] = "value: " . $event->value->getString();
        }
        if (!empty($event->currency)) {
            $contentFields[] = "currency: '" . $event->currency->getString() . "'";
        }
        if (!empty($event->content_name)) {
            $contentFields[] = "content_name: '" . $event->content_name->getString() . "'";
        }
        if (!empty($event->content_category)) {
            $contentFields[] = "content_category: '" . $event->content_category->getString() . "'";
        }

        return implode(', ', $contentFields);
    }

    /**
     * @param Purchase $event
     *
     * @return string
     */
    protected function renderPurchase(Purchase $event)
    {
        $contentFields = array();

        if (!empty($event->value)) {
            $contentFields[] = "value: " . $event->value->getString();
        }
        if (!empty($event->currency)) {
            $contentFields[] = "currency: '" . $event->currency->getString() . "'";
        }
        if (!empty($event->content_name)) {
            $contentFields[] = "content_name: '" . $event->content_name->getString() . "'";
        }
        if (!empty($event->content_type)) {
            $contentFields[] = "content_type: '" . $event->content_type->getString() . "'";
        }
        if (!empty($event->content_ids)) {
            $contentFields[] =
                "content_ids: [" . $this->getContentIdsCommaSeparated($event->content_ids) . "]";
        }
        if (!empty($event->num_items)) {
            $contentFields[] = "num_items: " . $event->num_items->getInt();
        }

        return implode(', ', $contentFields);
    }

    /**
     * @param Search $event
     *
     * @return string
     */
    protected function renderSearch(Search $event)
    {
        $contentFields = array();

        if (!empty($event->value)) {
            $contentFields[] = "value: " . $event->value->getString();
        }
        if (!empty($event->currency)) {
            $contentFields[] = "currency: '" . $event->currency->getString() . "'";
        }
        if (!empty($event->content_category)) {
            $contentFields[] = "content_category: '" . $event->content_category->getString() . "'";
        }
        if (!empty($event->content_ids)) {
            $contentFields[] =
                "content_ids: [" . $this->getContentIdsCommaSeparated($event->content_ids) . "]";
        }
        if (!empty($event->search_string)) {
            $contentFields[] = "search_string: '" . $event->search_string->getString() . "'";
        }

        return implode(', ', $contentFields);
    }

    /**
     * @param ViewContent $event
     *
     * @return string
     */
    protected function renderViewContent(ViewContent $event)
    {
        $contentFields = array();

        if (!empty($event->value)) {
            $contentFields[] = "value: " . $event->value->getString();
        }
        if (!empty($event->currency)) {
            $contentFields[] = "currency: '" . $event->currency->getString() . "'";
        }
        if (!empty($event->content_name)) {
            $contentFields[] = "content_name: '" . $event->content_name->getString() . "'";
        }
        if (!empty($event->content_type)) {
            $contentFields[] = "content_type: '" . $event->content_type->getString() . "'";
        }
        if (!empty($event->content_ids)) {
            $contentFields[] =
                "content_ids: [" . $this->getContentIdsCommaSeparated($event->content_ids) . "]";
        }

        return implode(', ', $contentFields);
    }

    /**
     * @param ContentId[] $contentIds
     *
     * @return string
     */
    protected function getContentIdsCommaSeparated($contentIds)
    {

        $contentIdsArray = array();
        foreach ($contentIds as $contentId) {
            $contentIdsArray[] = "'" . $contentId->getString() . "'";
        }

        return implode(',', $contentIdsArray);
    }

    /**
     * @return string
     */
    protected function getFacebookIncludeFunction()
    {
        return "!function(f,b,e,v,n,t,s){if(f.fbq)return;n=f.fbq=function(){n.callMethod?\n" .
            "n.callMethod.apply(n,arguments):n.queue.push(arguments)};if(!f._fbq)f._fbq=n;\n" .
            "n.push=n;n.loaded=!0;n.version='2.0';n.queue=[];t=b.createElement(e);t.async=!0;\n" .
            "t.src=v;s=b.getElementsByTagName(e)[0];s.parentNode.insertBefore(t,s)}(window,\n" .
            "document,'script','https://connect.facebook.net/en_US/fbevents.js');\n\n";
    }

    /**
     * @param int $facebookPixelId
     *
     * @return string
     */
    protected function getFacebookInitSnippet($facebookPixelId)
    {
        return "fbq('init', '" . $facebookPixelId . "');\n";
    }

    /**
     * @return string
     */
    protected function getOpenScriptTag()
    {
        return "<script>\n";
    }

    /**
     * @return string
     */
    protected function getCloseScriptTag()
    {
        return "</script>\n";
    }

    /**
     * @return string
     */
    protected function getOpenCommentTag()
    {
        return "<!-- Facebook Pixel Code -->\n";
    }

    /**
     * @return string
     */
    protected function getCloseCommentTag()
    {
        return '<!-- End Facebook Pixel Code -->';
    }

    /**
     * @param FacebookTrackingEvent[] $events
     *
     * @return string
     */
    protected function getNoScriptTagWithContent($events)
    {
        return "<noscript>{$this->noscriptContentRenderer->render($events)}</noscript>\n";
    }


}