<?php

namespace Shopgate\FacebookPixelLibrary\Tests\Unit\Setting\Factory;

use Shopgate\FacebookPixelLibrary\Setting\Entity\Details;
use Shopgate\FacebookPixelLibrary\Setting\Entity\EventSetting;
use Shopgate\FacebookPixelLibrary\Setting\Entity\SettingsSection;
use Shopgate\FacebookPixelLibrary\Setting\Factory\SettingsSectionDefaultFactory;
use Shopgate\FacebookPixelLibrary\Setting\Repository\Provider\Translation;
use Shopgate\FacebookPixelLibrary\Setting\ValueObject\Description;
use Shopgate\FacebookPixelLibrary\Setting\ValueObject\EventName;
use Shopgate\FacebookPixelLibrary\Setting\ValueObject\EventSettingFieldName;
use Shopgate\FacebookPixelLibrary\Setting\ValueObject\EventSettingLabel;
use Shopgate\FacebookPixelLibrary\Setting\ValueObject\Label;
use Shopgate\FacebookPixelLibrary\Setting\ValueObject\Link;
use Shopgate\FacebookPixelLibrary\Setting\ValueObject\LinkTitle;
use Shopgate\FacebookPixelLibrary\Setting\ValueObject\LinkUrl;
use Shopgate\FacebookPixelLibrary\Setting\ValueObject\Title;

class SettingsSectionDefaultFactoryTest extends \PHPUnit_Framework_TestCase
{
    /** @var SettingsSectionDefaultFactory */
    private $subjectUnderTest;

    /** @var Translation|\PHPUnit_Framework_MockObject_MockObject */
    private $translationRepository;

    public function setUp()
    {
        $this->translationRepository = $this
            ->getMockBuilder('Shopgate\FacebookPixelLibrary\Setting\Repository\Provider\Translation')
            ->getMockForAbstractClass()
        ;

        $this->subjectUnderTest = new SettingsSectionDefaultFactory($this->translationRepository);
    }

    public function testGetSettingsSectionForAllEvents()
    {
        $testDummyReturn = 'a settings section object we just pass through would usually be created here';

        // overwriting this because for this test we need to mock a method
        $this->subjectUnderTest = $this
            ->getMockBuilder('\Shopgate\FacebookPixelLibrary\Setting\Factory\SettingsSectionDefaultFactory')
            ->disableOriginalConstructor()
            ->setMethods(array('getSettingsSection'))
            ->getMock()
        ;

        $this->subjectUnderTest
            ->expects($this->once())
            ->method('getSettingsSection')
            ->with(
                array(
                    new EventName(EventName::VIEW_CONTENT),
                    new EventName(EventName::ADD_TO_CART),
                    new EventName(EventName::INITIATE_CHECKOUT),
                    new EventName(EventName::PURCHASE),
                    new EventName(EventName::SEARCH),
                    new EventName(EventName::ADD_TO_WISHLIST),
                    new EventName(EventName::ADD_PAYMENT_INFO),
                    new EventName(EventName::LEAD),
                    new EventName(EventName::COMPLETE_REGISTRATION),
                )
            )
            ->willReturn($testDummyReturn)
        ;

        $this->assertSame(
            $testDummyReturn,
            $this->subjectUnderTest->getSettingsSectionForAllEvents()
        );
    }

    /**
     * @param SettingsSection $expectedSettingsSection
     * @param EventName[]     $supportedEvents
     *
     * @dataProvider provideSettingsSectionFixtures
     */
    public function testGetSettingsSection(SettingsSection $expectedSettingsSection, array $supportedEvents)
    {
        $this->translationRepository
            ->expects($this->once())
            ->method('getSettingsSectionTitle')
            ->willReturn(new Title('title'))
        ;

        $this->translationRepository
            ->expects($this->once())
            ->method('getDetailsLabel')
            ->willReturn(new Label('details'))
        ;

        $this->translationRepository
            ->expects($this->once())
            ->method('getDetailsLink')
            ->willReturn(new Link(new LinkUrl('http://details.info'), new LinkTitle('details info')))
        ;

        $this->translationRepository
            ->expects($this->once())
            ->method('getDetailsDescription')
            ->willReturn(new Description('description'))
        ;

        $this
            ->translationRepository
            ->expects($this->exactly(count($supportedEvents)))
            ->method('getEventSettingActiveLabel')
            ->willReturn(new EventSettingLabel('active'))
        ;

        $this
            ->translationRepository
            ->expects($this->exactly(count($supportedEvents)))
            ->method('getEventSettingInactiveLabel')
            ->willReturn(new EventSettingLabel('inactive'))
        ;

        foreach ($supportedEvents as $supportedEvent) {
            $this
                ->translationRepository
                ->expects($this->any())
                ->method("getLabel{$supportedEvent->getString()}Setting")
                ->willReturn(new Label("{$supportedEvent->getString()} Label"))
            ;
        }

        $this->assertEquals(
            $expectedSettingsSection,
            $this->subjectUnderTest->getSettingsSection($supportedEvents)
        );
    }

    /**
     * @return array
     */
    public function provideSettingsSectionFixtures()
    {
        return array(
            'no events'                => array(
                new SettingsSection(
                    new Title('title'),
                    new Details(
                        new Label('details'),
                        new Link(new LinkUrl('http://details.info'), new LinkTitle('details info')),
                        new Description('description')
                    ),
                    array()
                ),
                array(),
            ),
            'ViewContent'              => array(
                new SettingsSection(
                    new Title('title'),
                    new Details(
                        new Label('details'),
                        new Link(new LinkUrl('http://details.info'), new LinkTitle('details info')),
                        new Description('description')
                    ),
                    array(
                        new EventSetting(
                            new EventSettingFieldName('facebook_pixel_view_content_event_active'),
                            new Label('ViewContent Label'),
                            new EventSettingLabel('active'),
                            new EventSettingLabel('inactive')
                        ),
                    )
                ),
                array(new EventName(EventName::VIEW_CONTENT)),
            ),
            'ViewContent, ViewContent' => array(
                new SettingsSection(
                    new Title('title'),
                    new Details(
                        new Label('details'),
                        new Link(new LinkUrl('http://details.info'), new LinkTitle('details info')),
                        new Description('description')
                    ),
                    array(
                        new EventSetting(
                            new EventSettingFieldName('facebook_pixel_view_content_event_active'),
                            new Label('ViewContent Label'),
                            new EventSettingLabel('active'),
                            new EventSettingLabel('inactive')
                        ),
                        new EventSetting(
                            new EventSettingFieldName('facebook_pixel_view_content_event_active'),
                            new Label('ViewContent Label'),
                            new EventSettingLabel('active'),
                            new EventSettingLabel('inactive')
                        ),
                    )
                ),
                array(new EventName(EventName::VIEW_CONTENT), new EventName(EventName::VIEW_CONTENT)),
            ),
            'AddToCart, ViewContent'   => array(
                new SettingsSection(
                    new Title('title'),
                    new Details(
                        new Label('details'),
                        new Link(new LinkUrl('http://details.info'), new LinkTitle('details info')),
                        new Description('description')
                    ),
                    array(
                        new EventSetting(
                            new EventSettingFieldName('facebook_pixel_add_to_cart_event_active'),
                            new Label('AddToCart Label'),
                            new EventSettingLabel('active'),
                            new EventSettingLabel('inactive')
                        ),
                        new EventSetting(
                            new EventSettingFieldName('facebook_pixel_view_content_event_active'),
                            new Label('ViewContent Label'),
                            new EventSettingLabel('active'),
                            new EventSettingLabel('inactive')
                        ),
                    )
                ),
                array(new EventName(EventName::ADD_TO_CART), new EventName(EventName::VIEW_CONTENT)),
            ),
            'all events'               => array(
                new SettingsSection(
                    new Title('title'),
                    new Details(
                        new Label('details'),
                        new Link(new LinkUrl('http://details.info'), new LinkTitle('details info')),
                        new Description('description')
                    ),
                    array(
                        new EventSetting(
                            new EventSettingFieldName('facebook_pixel_view_content_event_active'),
                            new Label('ViewContent Label'),
                            new EventSettingLabel('active'),
                            new EventSettingLabel('inactive')
                        ),
                        new EventSetting(
                            new EventSettingFieldName('facebook_pixel_add_to_cart_event_active'),
                            new Label('AddToCart Label'),
                            new EventSettingLabel('active'),
                            new EventSettingLabel('inactive')
                        ),
                        new EventSetting(
                            new EventSettingFieldName('facebook_pixel_initiate_checkout_event_active'),
                            new Label('InitiateCheckout Label'),
                            new EventSettingLabel('active'),
                            new EventSettingLabel('inactive')
                        ),
                        new EventSetting(
                            new EventSettingFieldName('facebook_pixel_purchase_event_active'),
                            new Label('Purchase Label'),
                            new EventSettingLabel('active'),
                            new EventSettingLabel('inactive')
                        ),
                        new EventSetting(
                            new EventSettingFieldName('facebook_pixel_search_event_active'),
                            new Label('Search Label'),
                            new EventSettingLabel('active'),
                            new EventSettingLabel('inactive')
                        ),
                        new EventSetting(
                            new EventSettingFieldName('facebook_pixel_add_to_wishlist_event_active'),
                            new Label('AddToWishlist Label'),
                            new EventSettingLabel('active'),
                            new EventSettingLabel('inactive')
                        ),
                        new EventSetting(
                            new EventSettingFieldName('facebook_pixel_add_payment_info_event_active'),
                            new Label('AddPaymentInfo Label'),
                            new EventSettingLabel('active'),
                            new EventSettingLabel('inactive')
                        ),
                        new EventSetting(
                            new EventSettingFieldName('facebook_pixel_lead_event_active'),
                            new Label('Lead Label'),
                            new EventSettingLabel('active'),
                            new EventSettingLabel('inactive')
                        ),
                        new EventSetting(
                            new EventSettingFieldName('facebook_pixel_complete_registration_event_active'),
                            new Label('CompleteRegistration Label'),
                            new EventSettingLabel('active'),
                            new EventSettingLabel('inactive')
                        ),
                    )
                ),
                array(
                    new EventName(EventName::VIEW_CONTENT),
                    new EventName(EventName::ADD_TO_CART),
                    new EventName(EventName::INITIATE_CHECKOUT),
                    new EventName(EventName::PURCHASE),
                    new EventName(EventName::SEARCH),
                    new EventName(EventName::ADD_TO_WISHLIST),
                    new EventName(EventName::ADD_PAYMENT_INFO),
                    new EventName(EventName::LEAD),
                    new EventName(EventName::COMPLETE_REGISTRATION),
                ),
            ),
        );
    }
}